#include "MyWindow.hpp"
#include "ColorDef.hpp"

#define X_MARGIN 8
#define Y_MARGIN 8
#define COLOR_MY_WINDOW_BACK (SFXRGBColor(0xCC, 0xFF, 0xCC, 0x00))

enum {
    ID_BUTTON1 = 1,
    ID_BUTTON2
};

// create MyWindow instance
MyWindowSmp MyWindow::NewInstance(SFCErrorPtr exception)
{
    // create MyWindow instance using Factory()
    // since Factory() returns SFYResponderSmp type
    // downcast using static_pointer_cast operator into MyWindowSmp type
    return static_pointer_cast<MyWindow>(Factory(:: new MyWindow, exception));
}

// constructor
MyWindow::MyWindow(Void) : _ifShowHw(true) static_throws
{
    if (static_try()) {

        // set responder type
        SetType(CODE_TYPE);

        // register key handler
        static_throw(RegisterHandler(
            SFXEventRange(SFEVT_KEY, SFEVT_KEY, SFP16_BEGIN, SFP16_END),
            XANDLER_INTERNAL(OnKey)
        ));
    }

    // since MyWindow::HandleRenderRequest function is called first when receiving drawing event
    // it is enough to describe drawing procedure of MyWindow in this function
    // it is unnecessary to define, implement, register drawing handler newly for MyWindow
    
    // set MyWindow's background color to light green
    SetBackgroundColor(COLOR_MY_WINDOW_BACK);
}

// destructor
MyWindow::~MyWindow(Void)
{
}

// make MyWindow: make various controls on MyWindow
SFCError MyWindow::Make(Void)
{
    SFXGrid offset(0, 0);
    SFCError error(SFERR_NO_ERROR);

    // set left-top of drawing region of MyWindow
    offset.Add(X_MARGIN, Y_MARGIN);

    // make button controls on MyWindow
    if ((error = CreateButtons(&offset)) != SFERR_NO_ERROR) {

        TRACE("button error");
        return error;
    }

    // make check box controls controls on MyWindow 
    if ((error = CreateCheckBoxes(&offset)) != SFERR_NO_ERROR) {

        TRACE("checkbox error");
        return error;
    }

    // make radio button controls on MyWindow
    if ((error = CreateRadioButtons(&offset)) != SFERR_NO_ERROR) {

        TRACE("radiobutton error");
        return error;
    }

    // make conmo box control on MyWindow
    if ((error = CreateComboBox(&offset)) != SFERR_NO_ERROR) {

        TRACE("combobox error");
        return error;
    }

    // make text label control and edit box control on MyWindow
    if ((error = CreatePassLine(&offset)) != SFERR_NO_ERROR) {

        TRACE("passline error");
        return error;
    }

    // make list box control on MyWindow
    if ((error = CreateListBox(&offset)) != SFERR_NO_ERROR) {

        TRACE("listbox error");
        return error;
    }

    return error;
}

// make button controls
SFCError MyWindow::CreateButtons(SFXGridPtr offs)
{
    SFXGraphicsPtr graphics = SFXGraphics::GetInstance();
    SInt16Const height = graphics->GetFontHeight() + 4;                 // button control's height
    SInt16Const width = (GetLocalBound().GetWidth() - 4 * X_MARGIN) / 3;// button control's width
    SFXRectangle rect(*offs, SFXSize(width, height));                   // button control 1's real region
    SFCError error(SFERR_NO_ERROR);

    // create and place button control 1
    if ((_button1 = SFZTextButtonControl::NewInstance(&error)) != null) {

        // set button control 1's parent responder to MyWindow
        error = _button1->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into button control 1
            error = _button1->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnButton12)
            );

            if (error == SFERR_NO_ERROR) {

                // set button control 1's label to "OK"
                error = _button1->SetText("OK");

                if (error == SFERR_NO_ERROR) {

                    // set button control 1's ID
                    _button1->SetID(ID_BUTTON1);

                    // set button control 1's real region 
                    _button1->SetRealBound(rect);

                    // set button control 1's state to "visible" + "active" + "enable" + "focus" together 
                    _button1->SetState(true, true, true, true);
                }
            }
        }
    }

    // create and place button control 2
    if ((_button2 = SFZTextButtonControl::NewInstance(&error)) != null) {

        // set button control 2's parent responder to MyWindow
        error = _button2->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into button control 2
            error = _button2->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnButton12)
            );

            if (error == SFERR_NO_ERROR) {

                // set button control 2's label to "NG" 
                error = _button2->SetText("NG");

                if (error == SFERR_NO_ERROR) {

                    // set button control 2's ID
                    _button2->SetID(ID_BUTTON2);

                    // set button control 2's real region 
                    rect.Offset(X_MARGIN + rect.GetWidth(), 0);
                    _button2->SetRealBound(rect);

                    // set button control 2's state to "visible" + "inactive" + "unenable" + "unfocus" together 
                    _button2->SetState(true, false, false, false);
                }
            }
        }
    }

    // create and place button control 3
    if ((_button3 = SFZTextButtonControl::NewInstance(&error)) != null) {

        // set button control 3's parent responder to MyWindow
        error = _button3->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into button control 3
            error = _button3->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnButton3)
            );

            if (error == SFERR_NO_ERROR) {

                // set button control 3's label to "Off"
                error = _button3->SetText("Off");

                if (error == SFERR_NO_ERROR) {

                    // calculate button control 3's real region
                    rect.Offset(X_MARGIN + rect.GetWidth(), 0);

                    // set button control 3's real region  
                    _button3->SetRealBound(rect);

                    // set button control 3's to state "visible" + "active" + "enable" + "unfocus" together 
                    _button3->SetState(true, true, true, false);
                }
            }
        }
    }

    // update offset
    offs->AddY(rect.GetHeight() + Y_MARGIN);

    return error;
}

// make a check box control
SFCError MyWindow::CreateCheckBoxes(SFXGridPtr offs)
{
    SFXGraphicsPtr graphics = SFXGraphics::GetInstance();
    SInt16Const height = graphics->GetFontHeight() + 4;                 // check box control's height
    SInt16Const width = (GetLocalBound().GetWidth() - 3 * X_MARGIN) / 2;// check box control's width
    SFXRectangle rect(*offs, width, height);                            // check box control 1's real region 
    SFCError error(SFERR_NO_ERROR);

    // make check box control 1
    if ((_checkbox1 = SFZCheckboxControl::NewInstance(&error)) != null) {

        // set check box control 1's parent responder to MyWindow
        error = _checkbox1->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into check box control 1
            error = _checkbox1->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnCheck)
            );

            if (error == SFERR_NO_ERROR) {

                // set check box control 1's label to "Hit"
                error = _checkbox1->SetText("Hit");

                if (error == SFERR_NO_ERROR) {

                    // set check box control 1's current value to "true"(i.e. check box control 1 is checked)
                    _checkbox1->SetCurrentValue(true);

                    // set check box control 1's horizontal alignment to "left-aligned"
                    _checkbox1->SetHorizontalAlign(SFZCheckboxControl::HORIZONTAL_LEFT);

                    // set check box control 1's real region 
                    _checkbox1->SetRealBound(rect);

                    // set check box control 1's state to "visible" + "active" + "enable" + "unfocus" together 
                    _checkbox1->SetState(true, true, true, false);
                }
            }
        }
    }

    // create and place check box control 2
    if ((_checkbox2 = SFZCheckboxControl::NewInstance(&error)) != null) {

        // set check box control 2's parent responder to MyWindow
        error = _checkbox2->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into check box control 2
            error = _checkbox2->RegisterHandler(
             SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
             XANDLER_INTERNAL(OnCheck)
            );

            // set check box control 2's label to "Run"
            error = _checkbox2->SetText("Run");

            if (error == SFERR_NO_ERROR) {

                // calculate check box control 2's real region
                rect.Offset(X_MARGIN + rect.GetWidth(), 0);

                // set check box control 2's horizontal alignment to "left-aligned"
                _checkbox2->SetHorizontalAlign(SFZCheckboxControl::HORIZONTAL_LEFT);

                // set check box control 2's real region 
                _checkbox2->SetRealBound(rect);

                // set check box control 2's state to "visible" + "active" + "enable" + "unfocus" together
                _checkbox2->SetState(true, true, true, false);
            }
        }
    }

    // update offset
    offs->AddY(rect.GetHeight() + Y_MARGIN);

    return error;
}

// make radio button control
SFCError MyWindow::CreateRadioButtons(SFXGridPtr offs)
{
    SFXGraphicsPtr graphics = SFXGraphics::GetInstance();
    SInt16Const height = graphics->GetFontHeight() + 4;                 // radio button control's height
    SInt16Const width = (GetLocalBound().GetWidth() - 3 * X_MARGIN) / 2;// radio button control's width
    SFXRectangle rect(*offs, width, height);                            // radio button control 1's real region
    SFCError error(SFERR_NO_ERROR);

    // create and place radio button control 1
    if ((_radio1 = SFZRadiobuttonControl::NewInstance(&error)) != null) {

        // set radio button control 1's parent responder to MyWindow
        error = _radio1->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into radio button control 1
            error = _radio1->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnRadio)
            );

            if (error == SFERR_NO_ERROR) {

                // set radio button control 1's label to "Fast"
                error = _radio1->SetText("Fast");

                if (error == SFERR_NO_ERROR) {

                    // set radio button control 1's current value to "true"(i.e. radio button control 1 is checked)
                    _radio1->SetCurrentValue(true);

                    // set radio button control 1's horizontal alignment to "left-aligned"
                    _radio1->SetHorizontalAlign(SFZRadiobuttonControl::HORIZONTAL_LEFT);

                    // set radio button control 1's real region
                    _radio1->SetRealBound(rect);

                    // set radio button control 1's state to "visible" + "active" + "enable" + "unfocus" together 
                    _radio1->SetState(true, true, true, false);
                }
            }
        }
    }

    // create and place radio button control 2
    if ((_radio2 = SFZRadiobuttonControl::NewInstance(&error)) != null) {

        // set radio button control 2's parent responder to MyWindow
        error = _radio2->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into radio button control 2
            error = _radio2->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnRadio)
            );

            if (error == SFERR_NO_ERROR) {

                // set radio button control 2's label to "Slow"
                error = _radio2->SetText("Slow");

                if (error == SFERR_NO_ERROR) {

                    // calculate radio button control 2's real region
                    rect.Offset(X_MARGIN + rect.GetWidth(), 0);

                    // set radio button control 2's horizontal alignment to "left-aligned"
                    _radio2->SetHorizontalAlign(SFZRadiobuttonControl::HORIZONTAL_LEFT);

                    // set radio button control 2's real region
                    _radio2->SetRealBound(rect);

                    // set radio button control 2's state to "visible" + "active" + "enable" + "unfocus" together 
                    _radio2->SetState(true, true, true, false);
                }
            }
        }
    }

    // group radio button control 1 and 2
    _radio1->Group(_radio2);

    // update offset
    offs->AddY(rect.GetHeight() + Y_MARGIN);

    return error;
}

// make combo box control
SFCError MyWindow::CreateComboBox(SFXGridPtr offs)
{
    SFXGraphicsPtr graphics = SFXGraphics::GetInstance();
    SInt16Const height = graphics->GetFontHeight() + 4;             // combo box control's height
    SInt16Const width = (GetLocalBound().GetWidth() - 2 * X_MARGIN);// combo box control's width
    SFXRectangle rect(*offs, width, height);                        // combo box control's real region
    SFCError error(SFERR_NO_ERROR);

    // make combo box control
    if ((_combobox = SFZComboBoxControl::NewInstance(&error)) != null) {

        // set combo box control's parent responder to MyWindow
        error = _combobox->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into combo box control
            error = _combobox->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnComboBox)
            );

            if (error == SFERR_NO_ERROR) {

                // register value handler into flex list box control
                error = _combobox->GetListBoxControl()->RegisterHandler(
                    SFXEventRange(SFEVT_RESPONDER_VALUE, SFEVT_RESPONDER_VALUE, SFP16_VALUE_CURRENT, SFP16_VALUE_CURRENT),
                    XANDLER_INTERNAL(OnComboListValue)
                );

                if (error == SFERR_NO_ERROR) {

                    // set values for combo box control's item
                    SFXWideString item[] = {"First", "Second", "Third", "Four", "Five", "Six", "Seven", "Eight", "Nine", "Ten"};

                    // append array of items on combo box control together
                    error = _combobox->InsertLast(item, lengthof(item));

                    if (error == SFERR_NO_ERROR) {

                        // set combo box control's real region
                        _combobox->SetRealBound(rect);

                        // set combo box control's state to "visible" + "active" + "enable" + "unfocus" together
                        _combobox->SetState(true, true, true, false);
                    }
                }
            }
        }
    }

    // update offset
    offs->AddY(rect.GetHeight() + Y_MARGIN);

    return error;
}

// make text label control and edit box control
SFCError MyWindow::CreatePassLine(SFXGridPtr offs)
{
    SFXRectangle rect;
    SFCError error(SFERR_NO_ERROR);

    // create and place text label control
    if ((_singleTextLabel = SFZSingleTextLabelControl::NewInstance(&error)) != null) {

        // set text label control's parent responder to MyWindow
        error = _singleTextLabel->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // set text label control's label to "PW:"
            error = _singleTextLabel->SetText("PW:");

            if (error == SFERR_NO_ERROR) {

                // calculate suitable real region for text label control
                rect.Set(*offs, _singleTextLabel->GetSuitableBound().GetSize());

                // set text label control's real region 
                _singleTextLabel->SetRealBound(rect);

                // set text label control's state to "visible" + "active" + "unenable" + "unfocus" together
                _singleTextLabel->SetState(true, true, false, false);
            }
        }
    }

    // create and place edit box control
    if ((_singleEditBox = SFZSingleEditBoxControl::NewInstance(&error)) != null) {

        // set edit box control's parent responder to MyWindow
        error = _singleEditBox->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // register result handler into edit box control
            error = _singleEditBox->RegisterHandler(
                SFXEventRange(SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END),
                XANDLER_INTERNAL(OnSingleEditBox)
            );

            if (error == SFERR_NO_ERROR) {

                // set edit box control's password mode to "true"
                error = _singleEditBox->SetPasswordMode(true);

                if (error == SFERR_NO_ERROR) {

                    // set edit box control's maximum input length to 10 characters
                    error = _singleEditBox->SetMaximumLength(10);

                    if (error == SFERR_NO_ERROR) {

                        // only aphabet and number can be inputted
                        _singleEditBox->SetInputMode(AEE_TM_LETTERS | AEE_TM_NUMBERS);

                        // set edit box control's horizontal alignment to "left-aligned"
                        _singleEditBox->SetHorizontalAlign(SFZSingleEditBoxControl::HORIZONTAL_LEFT);

                        // calculate edit box control's real region
                        rect.Offset(rect.GetWidth(), 0);
                        rect.SetRight(GetLocalBound().GetWidth() - X_MARGIN);

                        // set edit box control's real region
                        _singleEditBox->SetRealBound(rect);

                        // set edit box control's state to "visible" + "active" + "enable" + "unfocus" together
                        _singleEditBox->SetState(true, true, true, false);
                    }
                }
            }
        }
    }

    // update offset
    offs->AddY(rect.GetHeight() + Y_MARGIN);

    return error;
}

// make list box control
SFCError MyWindow::CreateListBox(SFXGridPtr offs)
{
    SFXEventRange::AtomRecConst range[] = {                         // event range for list box control's handler
        {SFEVT_RESPONDER_RESULT, SFEVT_RESPONDER_RESULT, SFP16_BEGIN, SFP16_END},
        {SFEVT_RESPONDER_VALUE, SFEVT_RESPONDER_VALUE, SFP16_VALUE_CURRENT, SFP16_VALUE_CURRENT}
    };
    SFYHandler::RuleRec rule[lengthof(range)];                      // handler rule
    SInt16Const width = (GetLocalBound().GetWidth() - 2 * X_MARGIN);// list box control's width
    SFXRectangle rect(*offs, width, 0);                             // list box control's real region
    SFCError error(SFERR_NO_ERROR);

    // create and place list box control
    if ((_listbox = SFZListBoxControl::NewInstance(&error)) != null) {

        // set list box control's parent responder to MyWindow
        error = _listbox->SetParent(GetThis());

        if (error == SFERR_NO_ERROR) {

            // set handler rule for list box control
            rule[0].spp = XANDLER_FUNCTION(OnListBox);              // set result handler of list box control
            rule[0].reference = this;                               // set pointer to MyWindow(contains OnListBox)
            rule[1].spp = XANDLER_FUNCTION(OnListValue);            // set value handler of list box control
            rule[1].reference = this;                               // set pointer to MyWindow(contains OnListValue)

            // register all handlers for list box control together
            error = _listbox->RegisterHandler(atomic_cast(range), rule, lengthof(range));

            if (error == SFERR_NO_ERROR) {

                // items to be displayed on list box control
                SFXWideString item[] = {"First", "Second", "Third", "Four", "Five", "Six", "Seven", "Eight", "Nine", "Ten"};

                for (SInt16 i = 0; i < lengthof(item); ++i) {

                    // append item on list box control
                    error = _listbox->InsertLast(item[i]);

                    if (error != SFERR_NO_ERROR) {
                        break;
                    }
                }
                if (error == SFERR_NO_ERROR) {

                    // set list box control's move-item-up key to LEFT key
                    _listbox->SetUpKey(AVK_LEFT);

                    // set list box control's move-item-down key to RIGHT key
                    _listbox->SetDownKey(AVK_RIGHT);

                    // set rect's height to 80
                    rect.SetHeight(80);

                    // set list box control's real region
                    // real region's height is adjusted with GetSuitableBound()
                    _listbox->SetRealBound(_listbox->GetSuitableBound(rect).SetWidth(rect.GetWidth()));

                    // set list box control's state to "visible" + "active" + "enable" + "unfocus" together 
                    _listbox->SetState(true, true, true, false);
                }
            }
        }
    }

    // update offset
    offs->AddY(rect.GetHeight() + Y_MARGIN);

    return error;
}

// drawing handler
Void MyWindow::HandleRenderRequest(SFXGraphicsPtr graphics) const
{
    // SFXGraphics instance for drawing is passed as 1st argument(graphics)
    // left-top of MyWindow's local region is origin of drawing region

    // MyWindow's local region is filled in light green set with SFYWidget::BackgroundColor in constructor

    // if value of _flagShowHw flag is true, display "Hello Window"
    if (_ifShowHw) {

        // draw "Hello Window" string in black color at center of MyWindow's local region
        // get MyWindow's local region using GetLocalBound()
        graphics->DrawSingleText("Hello Window", GetLocalBound(), COLOR_BLACK);
    }
    return;
}

// key handler (called when MyWindow is focused and some key is pressed) 
XANDLER_IMPLEMENT_BOOLEVENT(MyWindow, OnKey, invoker, event)
{
// invoker is the responder of SFYResponderPtr type which called this handler(in this case, MyWindow)
// event is key event of SFXEventConstRef type

    SFYResponderSmp parent;
    Bool result(false);

    unused(invoker);
    switch (event.GetP16()) {

        // when the CLEAR key is pressed
        case AVK_CLR:

            // terminate MyWindow
            Terminate();
 
            // after termination, MyWindow remains in heap memory
            // MyWindow will be released from heap memory when reference count becomes 0
            // since helloworld application class contains MyWindow
            // MyWindow will be released from heap memory when helloworld applets terminates

            result= true;
            break;
    }
    return result;
}

// implement result handler of button control 1 and 2
XANDLER_IMPLEMENT_VOIDRESULT(MyWindow, OnButton12, invoker, reason, result)
{
    unused(reason);
    unused(result);

    // get pressed button control
    // downcast into SFZTextButtonControlPtr type since invoker is of SFYResponderPtr type
    SFZTextButtonControlPtr pushed(static_cast<SFZTextButtonControlPtr>(invoker));

    // get unpressed button control
    UInt32 pairID = (pushed->GetID() == ID_BUTTON1) ? ID_BUTTON2 : ID_BUTTON1;
    SFZTextButtonControlSmp pair = static_pointer_cast<SFZTextButtonControl>(GetThis()->GetChildFront(pairID));

    // set pressed button control's label to "NG"
    pushed->SetText("NG");

    // set only active state of pressed button control to "false" (i.e. inactive) 
    // since active state has higher priority than enable or focus state
    pushed->SetStateActive(false);

    // set unpressed button control's label to "OK" 
    pair->SetText("OK");

    // set unpressed button control's state to "active" + "enable" + "focus" together
    pair->SetStateActive(true);
    pair->SetStateEnable(true);
    pair->SetStateFocus(true);

    return;
}

// implement result handler of button control 3
XANDLER_IMPLEMENT_VOIDRESULT(MyWindow, OnButton3, invoker, reason, result)
{
    unused(invoker);
    unused(reason);
    unused(result);

    // switch from "Off" to "On" vice versa for button control 3's label
    _ifShowHw = !_ifShowHw;
    _button3->SetText(_ifShowHw ? "Off" : "On");

    // register MyWindow as redrawing region
    Invalidate();

    // when specifying nothing as argument of Invalidate(), all MyWindow's real region will be registered as redrawing region
    // executing Invalidate(), MyWindow is registered as redrawing region into renderer booted up at end of event loop

    return;
}

// implement result handler of check box control
XANDLER_IMPLEMENT_VOIDRESULT(MyWindow, OnCheck, invoker, reason, result)
{
    unused(invoker);
    unused(reason);
    unused(result);

    // get label string
    SFXWideString label1(_checkbox1->GetText());
    SFXWideString label2(_checkbox2->GetText());

    // display label string and current value on BREW Output Window
    TRACE("%S: %d", label1.GetCString(), _checkbox1->GetCurrentValue());
    TRACE("%S: %d", label2.GetCString(), _checkbox2->GetCurrentValue());

    return;
}

// implement result handler of radio button control
XANDLER_IMPLEMENT_VOIDRESULT(MyWindow, OnRadio, invoker, reason, result)
{
    unused(invoker);
    unused(reason);
    unused(result);

    // get label string
    SFXWideString label1(_radio1->GetText());
    SFXWideString label2(_radio2->GetText());

    // display label string and current value on BREW Output Window
    TRACE("%S: %d", label1.GetCString(), _radio1->GetCurrentValue());
    TRACE("%S: %d", label2.GetCString(), _radio2->GetCurrentValue());

    return;
}

// implement result handler of combo box control
XANDLER_IMPLEMENT_VOIDRESULT(MyWindow, OnComboBox, invoker, reason, result)
{
    unused(invoker);
    unused(reason);

    if (reason == SFP16_RESULT_OK) {

        // get selected item's string
        // result: selected item number 
        SFXWideString string(_combobox->GetItemText(result));

        // display selected item's number and string on BREW Output Window
        TRACE("'CB%d: %S' is selected.", result, string.GetCString());

        // synchronize flex list box control with combo box control
        _listbox->SetCurrentValue(result);
    }
    return;
}

// implement value handler of flex list box control internally contained by combo box control
XANDLER_IMPLEMENT_VOIDVALUE(MyWindow, OnComboListValue, invoker, reason, value)
{
    unused(invoker);
    unused(reason);

    // display focused item's number on BREW Output Window
    // value: focused item number
    TRACE("'CB%d'", value);

    return;
}

// implement result handler of edit box control
XANDLER_IMPLEMENT_VOIDRESULT(MyWindow, OnSingleEditBox, invoker, reason, result)
{
    unused(invoker);
    unused(reason);
    unused(result);

    // compare string with true password
    if (_singleEditBox->GetText().Compare("pass") == 0) {

        TRACE("password: authenticated!");
    } else {
        TRACE("password: can't be authenticated...");
    }

    return;
}

// implement result handler of list box control
XANDLER_IMPLEMENT_VOIDRESULT(MyWindow, OnListBox, invoker, reason, result)
{
    unused(invoker);
    unused(reason);

    if (reason == SFP16_RESULT_OK) {

        // get string of selected item
        // result: number of selected item
        SFXWideString string(_listbox->GetItemText(result));

        // display selected item's number and string on BREW Output Window
        TRACE("'LB%d: %S' is selected.", result, string.GetCString());

        // synchronize list box control with combo box control
        _combobox->SetCurrentValue(result);
    }
    return;
}

// implement value handler of list box control
XANDLER_IMPLEMENT_VOIDVALUE(MyWindow, OnListValue, invoker, reason, value)
{
    unused(invoker);
    unused(reason);

    // display focused item's number on BREW Output Window
    // value: focused item number
    TRACE("'LB%d'", value);
    return;
}
